package cn.iocoder.yudao.module.bpm.service.oa.stamp;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.stamp.vo.OaStampPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.stamp.vo.OaStampRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.stamp.vo.OaStampSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.stamp.OaStampDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.stamp.OaStampMapper;
import cn.iocoder.yudao.module.bpm.enums.DictDataConstants;
import cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.bpm.framework.flowable.core.enums.BpmConstants;
import cn.iocoder.yudao.module.bpm.service.task.BpmTaskService;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_STAMP_NOT_EXISTS;

/**
 * 用印流程信息 Service 实现类
 *
 * @author dp
 */
@Service
@Validated
public class OaStampServiceImpl implements OaStampService {

    /**
     * 对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_stamp";


    @Resource
    private OaStampMapper oaStampMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmTaskService bpmTaskService;

    @Resource
    private TaskService taskService;

    @Resource
    private FileApi fileApi;

    @Resource
    private AdminUserApi adminUserApi;

    @Resource
    private DeptApi deptApi;

    @Override
    @Transactional
    public Long stagingOaStamp(OaStampSaveReqVO stagingReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        AdminUserRespDTO loginUser = adminUserApi.getUser(loginUserId);
        Objects.requireNonNull(loginUser, "登录用户不能为空");
        // 人信息
        AdminUserRespDTO employee = adminUserApi.getUser(loginUserId);

        OaStampDO oaStamp = BeanUtils.toBean(stagingReqVO, OaStampDO.class);
        if (StringUtils.isBlank(oaStamp.getStampUuid())) {
            // 创建单据uuid
            String uuid = IdUtil.fastSimpleUUID();
            oaStamp.setStampUuid(uuid);
        }
        if (employee != null) {
            oaStamp.setEmployeeId(loginUser.getId());
            oaStamp.setEmployeeName(loginUser.getNickname());
            oaStamp.setEmployeePhone(loginUser.getMobile());
            oaStamp.setDeptId(loginUser.getDeptId());
            oaStamp.setPosition("员工职位");
            oaStamp.setUserId(loginUser.getId());
        }
        oaStamp.setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_STAGING);
        oaStamp.setInfoSource("0");
        oaStamp.setApplyEmployeeId(loginUser.getId());
        oaStamp.setApplyEmployeeName(loginUser.getNickname());
        List<Long> stampSealId = stagingReqVO.getStampSealId();
        if (CollectionUtil.isNotEmpty(stampSealId)) {
            oaStamp.setStampSealId(stampSealId.stream().map(String::valueOf).collect(Collectors.joining(",")));
        } else {
            oaStamp.setStampSealId(null);
        }
        // 暂存不保存审批人信息
        oaStamp.setStartUserSelectAssignees(null);
        // 保存或更新表单信息
        if (oaStamp.getId() == null) {
            oaStampMapper.insert(oaStamp);
        } else {
            validateOaStampExists(oaStamp.getId());
            oaStampMapper.updateById(oaStamp);
        }
        // 保存业务uuid到附件中
        saveFileList(stagingReqVO.getFileIdList(), oaStamp.getStampUuid());

        return oaStamp.getId();
    }

    @Override
    @Transactional
    public Long commitOaStamp(OaStampSaveReqVO commitReqVO) {
        if (CollectionUtil.isEmpty(commitReqVO.getStartUserSelectAssignees())) {
            throw exception(ErrorCodeConstants.TASK_CREATE_FAIL_NO_START_SELECT_ASSIGNEE);
        }
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        AdminUserRespDTO loginUser = adminUserApi.getUser(loginUserId);
        if (loginUser == null) {
            throw exception(ErrorCodeConstants.OA_LOGIN_USER_NOT_EXISTS);
        }
        // 人信息
        AdminUserRespDTO employee = adminUserApi.getUser(loginUserId);
        if (employee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }


        OaStampDO oaStamp = BeanUtils.toBean(commitReqVO, OaStampDO.class);
        if (StringUtils.isBlank(oaStamp.getStampUuid())) {
            // 创建单据uuid
            String uuid = IdUtil.fastSimpleUUID();
            oaStamp.setStampUuid(uuid);
        }
        oaStamp.setEmployeeId(employee.getId());
        oaStamp.setEmployeeName(employee.getNickname());
        oaStamp.setEmployeePhone(employee.getMobile());
        oaStamp.setDeptId(employee.getDeptId());
        oaStamp.setPosition("员工职位");
        oaStamp.setUserId(loginUser.getId());
        oaStamp.setInfoSource("0");
        oaStamp.setApplyEmployeeId(loginUser.getId());
        oaStamp.setApplyEmployeeName(loginUser.getNickname());
        List<Long> stampSealId = commitReqVO.getStampSealId();
        if (CollectionUtil.isNotEmpty(stampSealId)) {
            oaStamp.setStampSealId(stampSealId.stream().map(String::valueOf).collect(Collectors.joining(",")));
        } else {
            oaStamp.setStampSealId(null);
        }
        // 保存或更新表单信息
        if (oaStamp.getId() == null) {
            oaStampMapper.insert(oaStamp);
        } else {
            OaStampDO oaStampDO = validateOaStampExists(oaStamp.getId());
            if (StrUtil.isNotBlank(oaStampDO.getProcInstId())) {
                throw exception(ErrorCodeConstants.PROCESS_INSTANCE_CREATE_FAIL_HAS_PROCESS);
            }
            oaStampMapper.updateById(oaStamp);
        }
        // 发起流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        processInstanceVariables.put("auditPass", "true");
        // 添加审批人信息
        Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
        List<Long> selectAssignees = commitReqVO.getStartUserSelectAssignees();
        startUserSelectAssignees.put("approver", selectAssignees);
        String processInstanceId = processInstanceApi
                .createProcessInstance(loginUser.getId(),
                        new BpmProcessInstanceCreateReqDTO()
                                .setProcessDefinitionKey(PROCESS_KEY)
                                .setVariables(processInstanceVariables)
                                .setBusinessKey(String.valueOf(oaStamp.getId()))
                                .setStartUserSelectAssignees(startUserSelectAssignees));

        //获取下一个审批人
        Task task = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();

        // 审批同意
        BpmTaskApproveReqVO agreeReqVO = new BpmTaskApproveReqVO();
        agreeReqVO.setId(task.getId());
        agreeReqVO.setReason("[首次提交]");
        agreeReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_COMMITTED));
        taskService.setVariable(task.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_COMMITTED);
        taskService.setVariable(task.getId(), "auditPass", "true");
        bpmTaskService.approveTask(loginUserId, agreeReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();
        if (nextTask == null) {
            throw exception(ErrorCodeConstants.TASK_CREATE_FAIL_NO_START_SELECT_ASSIGNEE);
        }
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        AdminUserRespDTO currentAuditEmployee = adminUserApi.getUser(currentAuditEmployeeId);


        // 将工作流的流程实例ID、单据状态、最后一次审批时间、当前审批人更新到单据信息中
        oaStampMapper.updateById(new OaStampDO()
                .setId(oaStamp.getId())
                .setProcInstId(processInstanceId)
                .setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_COMMITTED)
                .setFinalAuditDate(LocalDateTime.now())
                .setCurrentAuditEmployeeId(currentAuditEmployeeId)
                .setCurrentAuditEmployeeName(currentAuditEmployee.getNickname())
                .setStartUserSelectAssignees(selectAssignees.stream().map(String::valueOf).collect(Collectors.joining(","))));
        // 保存业务uuid到附件中
        saveFileList(commitReqVO.getFileIdList(), oaStamp.getStampUuid());
        // TODO DP 发送提交成功站内信
        return oaStamp.getId();
    }

    @Override
    @Transactional
    public Long agreeOaStamp(BpmTaskApproveReqVO agreeReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // TODO DP 根据登录人查询出对应的员工信息

        Task currentTask = bpmTaskService.getTask(agreeReqVO.getId());
        if (currentTask == null) {
            throw exception(ErrorCodeConstants.TASK_NOT_EXISTS);
        }
        // 先更新为审批中
        taskService.setVariable(currentTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_AUDITING);
        LambdaQueryWrapper<OaStampDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(OaStampDO::getProcInstId, currentTask.getProcessInstanceId());
        OaStampDO oaStamp = oaStampMapper.selectOne(lambdaQueryWrapper);
        // 如果是最后一个人审批同意，设置流程审批状态为已审核
        String[] auditPersons = oaStamp.getStartUserSelectAssignees().split(",");
        String lastAuditPerson = Arrays.stream(auditPersons)
                .reduce((first, second) -> second)
                .orElse(null);
        if (currentTask.getAssignee().equals(lastAuditPerson)) {
            // 更新为已审核(暂时未生效，未找到原因，先不处理)
            taskService.setVariable(agreeReqVO.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_AUDITED);
        }
        // 审批同意
        String reason = "[同意]";
        if (StrUtil.isNotBlank(agreeReqVO.getReason())) {
            reason = reason + agreeReqVO.getReason();
        }
        agreeReqVO.setReason(reason);
        agreeReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_AUDITING));
        taskService.setVariable(agreeReqVO.getId(), "auditPass", "true");
        bpmTaskService.approveTask(loginUserId, agreeReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        OaStampDO oaStampDO = new OaStampDO();
        if (nextTask != null) {
            Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
            AdminUserRespDTO currentAuditEmployee = adminUserApi.getUser(currentAuditEmployeeId);

            // 如果审批人重复，实际未结束，再次更新为审核中
            taskService.setVariable(nextTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_AUDITING);
            oaStampDO.setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_AUDITING)
                    .setCurrentAuditEmployeeId(currentAuditEmployeeId)
                    .setCurrentAuditEmployeeName(currentAuditEmployee.getNickname())
                    .setFinalAuditDate(LocalDateTime.now())
                    .setId(oaStamp.getId());

        } else {
            oaStampDO.setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_AUDITED)
                    .setCurrentAuditEmployeeId(null)
                    .setCurrentAuditEmployeeName(null)
                    .setFinalAuditDate(LocalDateTime.now())
                    .setId(oaStamp.getId());

            // TODO 复制业务单据信息到业务模块单据表

        }
        oaStampMapper.updateById(oaStampDO);
        // 发送通知

        // 返回
        return 1L;
    }

    @Override
    @Transactional
    public Long disagreeOaStamp(BpmTaskReturnReqVO disagreeReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // TODO DP 根据登录人查询出对应的员工信息

        Task currentTask = bpmTaskService.getTask(disagreeReqVO.getId());

        disagreeReqVO.setTargetTaskDefinitionKey("modifyApply");
        disagreeReqVO.setReason("[驳回]" + disagreeReqVO.getReason());
        disagreeReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_RETURNED));
        taskService.setVariable(disagreeReqVO.getId(), "auditPass", "false");
        taskService.setVariable(disagreeReqVO.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_RETURNED);
        bpmTaskService.returnTask(loginUserId, disagreeReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        AdminUserRespDTO currentAuditEmployee = adminUserApi.getUser(currentAuditEmployeeId);

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaStampDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaStampDO::getCurrentAuditEmployeeId, currentAuditEmployeeId)
                .set(OaStampDO::getCurrentAuditEmployeeName, currentAuditEmployee.getNickname())
                .set(OaStampDO::getFinalAuditDate, LocalDateTime.now())
                .set(OaStampDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_RETURNED)
                .eq(OaStampDO::getProcInstId, currentTask.getProcessInstanceId());
        oaStampMapper.update(updateWrapper);

        // 发送通知

        return 1L;
    }

    @Override
    @Transactional
    public Long revocationOaStamp(BpmTaskApproveReqVO revocationReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // TODO DP 根据登录人查询出对应的员工信息

        Task currentTask = bpmTaskService.getTask(revocationReqVO.getId());
        if (currentTask == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.TASK_NOT_EXISTS);
        }
        LambdaQueryWrapper<OaStampDO> lambdaQueryWrapper = new LambdaQueryWrapper<OaStampDO>()
                .eq(OaStampDO::getProcInstId, currentTask.getProcessInstanceId());
        OaStampDO oaStampDO = oaStampMapper.selectOne(lambdaQueryWrapper);
        if (!DictDataConstants.OA_AUDIT_STATUS_COMMITTED.equals(oaStampDO.getAuditStatus())
                || !Objects.equals(String.valueOf(loginUserId), oaStampDO.getCreator())) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.TASK_REVOCATION_NOT_ALLOWED);
        }

        BpmTaskReturnReqVO returnReqVO = BeanUtils.toBean(revocationReqVO, BpmTaskReturnReqVO.class);
        returnReqVO.setTargetTaskDefinitionKey("modifyApply");
        String reason = "[撤回]";
        if (StrUtil.isNotBlank(returnReqVO.getReason())) {
            reason = reason + returnReqVO.getReason();
        }
        returnReqVO.setReason(reason);
        returnReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_RECALLED));
        taskService.setVariable(currentTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_RECALLED);
        bpmTaskService.revocationTask(loginUserId, returnReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        AdminUserRespDTO currentAuditEmployee = adminUserApi.getUser(currentAuditEmployeeId);

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaStampDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaStampDO::getCurrentAuditEmployeeId, currentAuditEmployeeId)
                .set(OaStampDO::getCurrentAuditEmployeeName, currentAuditEmployee.getNickname())
                .set(OaStampDO::getFinalAuditDate, LocalDateTime.now())
                .set(OaStampDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_RECALLED)
                .eq(OaStampDO::getProcInstId, currentTask.getProcessInstanceId());
        oaStampMapper.update(updateWrapper);

        // 发送通知

        return 1L;
    }

    @Override
    @Transactional
    public Long reCommitOaStamp(OaStampSaveReqVO reCommitReqVO) {
        if (reCommitReqVO.getId() == null) {
            throw exception(ErrorCodeConstants.SERVICE_ID_NOT_EXISTS);
        }
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        AdminUserRespDTO loginUser = adminUserApi.getUser(loginUserId);
        if (loginUser == null) {
            throw exception(ErrorCodeConstants.OA_LOGIN_USER_NOT_EXISTS);
        }
        // 人信息
        AdminUserRespDTO employee = adminUserApi.getUser(loginUserId);
        if (employee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }

        OaStampDO oaStampNew = BeanUtils.toBean(reCommitReqVO, OaStampDO.class);
        OaStampDO oaStampOld = validateOaStampExists(oaStampNew.getId());
        oaStampNew.setStampSealId(oaStampOld.getStampUuid())
                .setProcInstId(oaStampOld.getProcInstId());

        oaStampNew.setEmployeeId(employee.getId());
        oaStampNew.setEmployeeName(employee.getNickname());
        oaStampNew.setEmployeePhone(employee.getMobile());
        oaStampNew.setDeptId(employee.getDeptId());
        oaStampNew.setPosition("员工职位");
        oaStampNew.setUserId(loginUser.getId());
        oaStampNew.setApplyEmployeeId(loginUser.getId());
        oaStampNew.setApplyEmployeeName(loginUser.getNickname());
        List<Long> stampSealId = reCommitReqVO.getStampSealId();
        if (CollectionUtil.isNotEmpty(stampSealId)) {
            oaStampNew.setStampSealId(stampSealId.stream().map(String::valueOf).collect(Collectors.joining(",")));
        } else {
            oaStampNew.setStampSealId(null);
        }

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaStampNew.getProcInstId()).singleResult();
        BpmTaskApproveReqVO approveReqVO = new BpmTaskApproveReqVO();
        approveReqVO.setId(currentTask.getId())
                .setReason("[再次提交]");
        approveReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_COMMITTED));
        if (CollectionUtil.isNotEmpty(reCommitReqVO.getStartUserSelectAssignees())) {
            // 添加审批人信息到流程参数中
            Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
            List<Long> selectAssignees = reCommitReqVO.getStartUserSelectAssignees();
            startUserSelectAssignees.put("approver", selectAssignees);
            taskService.setVariable(currentTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_START_USER_SELECT_ASSIGNEES, startUserSelectAssignees);
            // 保存审批人信息
            oaStampNew.setStartUserSelectAssignees(selectAssignees.stream().map(String::valueOf).collect(Collectors.joining(",")));
        }
        taskService.setVariable(currentTask.getId(), "auditPass", "true");
        taskService.setVariable(currentTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_COMMITTED);
        // 再次提交，和审批通过逻辑相同
        bpmTaskService.approveTask(loginUserId, approveReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        AdminUserRespDTO currentAuditEmployee = adminUserApi.getUser(currentAuditEmployeeId);
        oaStampNew.setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_COMMITTED)
                .setCurrentAuditEmployeeId(currentAuditEmployeeId)
                .setCurrentAuditEmployeeName(currentAuditEmployee.getNickname())
                .setFinalAuditDate(LocalDateTime.now());
        oaStampMapper.updateById(oaStampNew);

        // 保存业务uuid到附件中
        saveFileList(reCommitReqVO.getFileIdList(), oaStampNew.getStampUuid());

        return 1L;
    }

    @Override
    @Transactional
    public Long closeOaStamp(Long id) {
        if (id == null) {
            throw exception(ErrorCodeConstants.SERVICE_ID_NOT_EXISTS);
        }
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // TODO DP 根据登录人查询出对应的员工信息

        OaStampDO oaStampDO = oaStampMapper.selectById(id);
        if (!DictDataConstants.OA_AUDIT_STATUS_RETURNED.equals(oaStampDO.getAuditStatus())
                && !DictDataConstants.OA_AUDIT_STATUS_RECALLED.equals(oaStampDO.getAuditStatus())) {
            throw exception(ErrorCodeConstants.TASK_CLOSE_NOT_ALLOWED);
        }

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaStampDO.getProcInstId()).singleResult();
        BpmTaskApproveReqVO approveReqVO = new BpmTaskApproveReqVO();
        approveReqVO.setId(currentTask.getId())
                .setReason("[关闭]");
        approveReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_CLOSED));
        taskService.setVariable(currentTask.getId(), "auditPass", "false");
        taskService.setVariable(currentTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_CLOSED);
        // 关闭，使用审批通过的方法实现
        bpmTaskService.approveTask(loginUserId, approveReqVO);

        LambdaUpdateWrapper<OaStampDO> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        lambdaUpdateWrapper.set(OaStampDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_CLOSED)
                .set(OaStampDO::getCurrentAuditEmployeeId, null)
                .set(OaStampDO::getCurrentAuditEmployeeName, null)
                .set(OaStampDO::getFinalAuditDate, LocalDateTime.now())
                .eq(OaStampDO::getId, id);
        oaStampMapper.update(lambdaUpdateWrapper);
        return 1L;
    }

    @Override
    public void deleteOaStamp(Long id) {
        // 校验存在
        validateOaStampDelete(id);
        // 删除
        oaStampMapper.deleteById(id);
    }

    @Override
    public OaStampRespVO getOaStamp(Long id) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // TODO DP 根据登录人查询出对应的员工信息

        OaStampDO oaStampDO = validateOaStampExists(id);
        OaStampRespVO oaStampRespVO = BeanUtils.toBean(oaStampDO, OaStampRespVO.class);
        if (oaStampDO.getDeptId() != null) {
            DeptRespDTO dept = deptApi.getDept(oaStampDO.getDeptId());
            if (dept != null) {
                oaStampRespVO.setDeptName(dept.getName());
            }
        }

        // 印章列表
        String stampSealId = oaStampDO.getStampSealId();
        if (StrUtil.isNotBlank(stampSealId)) {
            String[] stampSealIdArray = stampSealId.split(",");
            List<Long> stampSealIdList = Arrays.stream(stampSealIdArray).map(Long::valueOf).collect(Collectors.toList());
            oaStampRespVO.setStampSealId(stampSealIdList);
        } else {
            oaStampRespVO.setStampSealId(Collections.emptyList());
        }

        String procInstId = oaStampDO.getProcInstId();
        if (StrUtil.isNotBlank(procInstId)) {
            Task task = taskService.createTaskQuery()
                    .processInstanceId(procInstId)
                    .taskAssignee(String.valueOf(loginUserId))
                    .singleResult();
            if (DictDataConstants.OA_AUDIT_STATUS_COMMITTED.equals(oaStampDO.getAuditStatus())) {
                // 如果是已提交，不限制任务处理人是当前登录人，用于直接撤回
                task = taskService.createTaskQuery()
                        .processInstanceId(procInstId)
                        .singleResult();
            }
            if (task != null) {
                oaStampRespVO.setTaskId(task.getId());
            }
        }

        // 附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(oaStampDO.getStampUuid());
        oaStampRespVO.setFileList(fileList);

        // 审批记录
        if (StrUtil.isNotBlank(oaStampDO.getProcInstId())) {
            List<BpmTaskRespVO> auditRecordList = bpmTaskService.getAuditRecordListByProcessInstanceId(oaStampDO.getProcInstId());
            oaStampRespVO.setAuditRecordList(auditRecordList);
        }

        // 审批人
        String startUserSelectAssignees = oaStampDO.getStartUserSelectAssignees();
        if (StrUtil.isNotEmpty(startUserSelectAssignees)) {
            List<Long> userIdList = Arrays.stream(startUserSelectAssignees.split(","))
                    .map(Long::valueOf).collect(Collectors.toList());
            List<AdminUserRespDTO> userList = adminUserApi.getUserList(userIdList);
            List<AdminUserRespDTO> auditUserList = new ArrayList<>();
            for (Long userId : userIdList) {
                for (AdminUserRespDTO adminUserRespDTO : userList) {
                    if (Objects.equals(userId, adminUserRespDTO.getId())) {
                        auditUserList.add(adminUserRespDTO);
                        break;
                    }
                }
            }
            oaStampRespVO.setAuditUserList(auditUserList);
        }

        return oaStampRespVO;
    }

    @Override
    public OaStampRespVO getOaStampByProcInstId(String procInstId) {
        OaStampDO oaStampDO = oaStampMapper.selectOne(OaStampDO::getProcInstId, procInstId);
        OaStampRespVO oaStampRespVO = BeanUtils.toBean(oaStampDO, OaStampRespVO.class);
        if (oaStampDO.getDeptId() != null) {
            DeptRespDTO dept = deptApi.getDept(oaStampDO.getDeptId());
            if (dept != null) {
                oaStampRespVO.setDeptName(dept.getName());
            }
        }

        // 印章列表
        String stampSealId = oaStampDO.getStampSealId();
        if (StrUtil.isNotBlank(stampSealId)) {
            String[] stampSealIdArray = stampSealId.split(",");
            List<Long> stampSealIdList = Arrays.stream(stampSealIdArray).map(Long::valueOf).collect(Collectors.toList());
            oaStampRespVO.setStampSealId(stampSealIdList);
        } else {
            oaStampRespVO.setStampSealId(Collections.emptyList());
        }

        // 附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(oaStampDO.getStampUuid());
        oaStampRespVO.setFileList(fileList);

        // 审批记录
        if (StrUtil.isNotBlank(oaStampDO.getProcInstId())) {
            List<BpmTaskRespVO> auditRecordList = bpmTaskService.getAuditRecordListByProcessInstanceId(oaStampDO.getProcInstId());
            oaStampRespVO.setAuditRecordList(auditRecordList);
        }

        // 审批人
        String startUserSelectAssignees = oaStampDO.getStartUserSelectAssignees();
        if (StrUtil.isNotEmpty(startUserSelectAssignees)) {
            List<Long> userIdList = Arrays.stream(startUserSelectAssignees.split(","))
                    .map(Long::valueOf).collect(Collectors.toList());
            List<AdminUserRespDTO> userList = adminUserApi.getUserList(userIdList);
            List<AdminUserRespDTO> auditUserList = new ArrayList<>();
            for (Long userId : userIdList) {
                for (AdminUserRespDTO adminUserRespDTO : userList) {
                    if (Objects.equals(userId, adminUserRespDTO.getId())) {
                        auditUserList.add(adminUserRespDTO);
                        break;
                    }
                }
            }
            oaStampRespVO.setAuditUserList(auditUserList);
        }

        return oaStampRespVO;
    }

    @Override
    public PageResult<OaStampRespVO> getOaStampPage(OaStampPageReqVO pageReqVO) {
        PageResult<OaStampDO> oaStampDOPageResult = oaStampMapper.selectPage(pageReqVO);
        PageResult<OaStampRespVO> oaStampRespVOPageResult = BeanUtils.toBean(oaStampDOPageResult, OaStampRespVO.class);

        List<OaStampRespVO> oaStampRespVOList = oaStampRespVOPageResult.getList();
        if (CollectionUtil.isNotEmpty(oaStampRespVOList)) {
            List<Long> employeeIdList = oaStampRespVOList.stream().map(OaStampRespVO::getCurrentAuditEmployeeId).collect(Collectors.toList());
            List<AdminUserRespDTO> employeeList = adminUserApi.getUserList(employeeIdList);
            for (OaStampRespVO respVO : oaStampRespVOList) {
                for (AdminUserRespDTO employee : employeeList) {
                    if (employee.getId() != null && employee.getId().equals(respVO.getCurrentAuditEmployeeId())) {
                        respVO.setCurrentAuditEmployeeName(employee.getNickname());
                        break;
                    }
                }
            }

            List<Long> deptIdList = oaStampRespVOList.stream().map(OaStampRespVO::getDeptId).collect(Collectors.toList());
            List<DeptRespDTO> deptList = deptApi.getDeptList(deptIdList);
            for (OaStampRespVO respVO : oaStampRespVOList) {
                for (DeptRespDTO dept : deptList) {
                    if (dept.getId() != null && dept.getId().equals(respVO.getDeptId())) {
                        respVO.setDeptName(dept.getName());
                        break;
                    }
                }
            }
        }

        return oaStampRespVOPageResult;
    }

    @Override
    public PageResult<OaStampRespVO> exportOaStampExcel(OaStampPageReqVO pageReqVO) {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        return getOaStampPage(pageReqVO);
    }

    private void validateOaStampDelete(Long id) {
        OaStampDO oaStampDO = validateOaStampExists(id);
        if (!DictDataConstants.OA_AUDIT_STATUS_STAGING.equals(oaStampDO.getAuditStatus())) {
            throw exception(ErrorCodeConstants.DELETE_FAIL_NOT_STAGING);
        }
    }

    private OaStampDO validateOaStampExists(Long id) {
        OaStampDO oaStampDO = oaStampMapper.selectById(id);
        if (oaStampDO == null) {
            throw exception(OA_STAMP_NOT_EXISTS);
        }
        return oaStampDO;
    }

    /**
     * 保存业务uuid到附件中
     */
    private void saveFileList(List<Long> fileIdList, String serviceId) {
        if (CollectionUtil.isNotEmpty(fileIdList) && StrUtil.isNotEmpty(serviceId)) {
            fileApi.updateFileBiz(fileIdList, serviceId);
        }
    }

}