package cn.iocoder.yudao.module.expense.controller.admin.expenseitem;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.expense.controller.admin.expenseitem.vo.ExpenseItemPageReqVO;
import cn.iocoder.yudao.module.expense.controller.admin.expenseitem.vo.ExpenseItemRespVO;
import cn.iocoder.yudao.module.expense.controller.admin.expenseitem.vo.ExpenseItemSaveReqVO;
import cn.iocoder.yudao.module.expense.dal.dataobject.expenseitem.ExpenseItemDO;
import cn.iocoder.yudao.module.expense.service.expenseitem.ExpenseItemService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 报销费用项目信息")
@RestController
@RequestMapping("/expense/item")
@Validated
public class ExpenseItemController {

    @Resource
    private ExpenseItemService itemService;

    @PostMapping("/create")
    @Operation(summary = "创建报销费用项目信息")
    // @PreAuthorize("@ss.hasPermission('expense:item:create')")
    public CommonResult<Long> createItem(@Valid @RequestBody ExpenseItemSaveReqVO createReqVO) {
        return success(itemService.createItem(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新报销费用项目信息")
    // @PreAuthorize("@ss.hasPermission('expense:item:update')")
    public CommonResult<Boolean> updateItem(@Valid @RequestBody ExpenseItemSaveReqVO updateReqVO) {
        itemService.updateItem(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除报销费用项目信息")
    @Parameter(name = "id", description = "编号", required = true)
    // @PreAuthorize("@ss.hasPermission('expense:item:delete')")
    public CommonResult<Boolean> deleteItem(@RequestParam("id") Long id) {
        itemService.deleteItem(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得报销费用项目信息")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    // @PreAuthorize("@ss.hasPermission('expense:item:query')")
    public CommonResult<ExpenseItemRespVO> getItem(@RequestParam("id") Long id) {
        ExpenseItemDO item = itemService.getItem(id);
        return success(BeanUtils.toBean(item, ExpenseItemRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得报销费用项目信息分页")
    // @PreAuthorize("@ss.hasPermission('expense:item:query')")
    public CommonResult<PageResult<ExpenseItemRespVO>> getItemPage(@Valid ExpenseItemPageReqVO pageReqVO) {
        PageResult<ExpenseItemDO> pageResult = itemService.getItemPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, ExpenseItemRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出报销费用项目信息 Excel")
    // @PreAuthorize("@ss.hasPermission('expense:item:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportItemExcel(@Valid ExpenseItemPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<ExpenseItemDO> list = itemService.getItemPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "报销费用项目信息.xls", "数据", ExpenseItemRespVO.class,
                        BeanUtils.toBean(list, ExpenseItemRespVO.class));
    }

}