package cn.iocoder.yudao.module.expense.service.expenseitem;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.expense.controller.admin.expenseitem.vo.ExpenseItemPageReqVO;
import cn.iocoder.yudao.module.expense.controller.admin.expenseitem.vo.ExpenseItemSaveReqVO;
import cn.iocoder.yudao.module.expense.dal.dataobject.expenseitem.ExpenseItemDO;
import cn.iocoder.yudao.module.expense.dal.mysql.expenseitem.ExpenseItemMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.expense.enums.ErrorCodeConstants.EXPENSE_ITEM_NOT_EXISTS;

/**
 * 报销费用项目信息 Service 实现类
 *
 * @author dp
 */
@Service
@Validated
public class ExpenseItemServiceImpl implements ExpenseItemService {

    @Resource
    private ExpenseItemMapper itemMapper;

    @Override
    public Long createItem(ExpenseItemSaveReqVO createReqVO) {
        // 插入
        ExpenseItemDO item = BeanUtils.toBean(createReqVO, ExpenseItemDO.class);
        item.setExpenseItemUuid(IdUtil.fastSimpleUUID());
        itemMapper.insert(item);
        // 返回
        return item.getId();
    }

    @Override
    public void updateItem(ExpenseItemSaveReqVO updateReqVO) {
        // 校验存在
        validateItemExists(updateReqVO.getId());
        // 更新
        ExpenseItemDO updateObj = BeanUtils.toBean(updateReqVO, ExpenseItemDO.class);
        itemMapper.updateById(updateObj);
    }

    @Override
    public void deleteItem(Long id) {
        // 校验存在
        validateItemExists(id);
        // 删除
        itemMapper.deleteById(id);
    }

    private void validateItemExists(Long id) {
        if (itemMapper.selectById(id) == null) {
            throw exception(EXPENSE_ITEM_NOT_EXISTS);
        }
    }

    @Override
    public ExpenseItemDO getItem(Long id) {
        return itemMapper.selectById(id);
    }

    @Override
    public PageResult<ExpenseItemDO> getItemPage(ExpenseItemPageReqVO pageReqVO) {
        return itemMapper.selectPage(pageReqVO);
    }

    @Override
    public List<ExpenseItemDO> getExpenseItemList(Collection<Long> ids) {
        if (CollectionUtil.isEmpty(ids)) {
            return Collections.emptyList();
        }
        return itemMapper.selectBatchIds(ids);
    }

}