package cn.iocoder.yudao.module.employee.api;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.employee.api.dto.EmployeeCreateReqDTO;
import cn.iocoder.yudao.module.employee.api.dto.EmployeePageReqDTO;
import cn.iocoder.yudao.module.employee.api.dto.EmployeeQueryReqDTO;
import cn.iocoder.yudao.module.employee.api.dto.EmployeeRespDTO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 员工 API 接口
 *
 * @author zhaopq
 */
public interface EmployeeApi {

    /**
     * 根据id查询员工信息
     *
     * @param id 用户ID
     * @return 员工对象信息
     */
    EmployeeRespDTO getEmployeeById(Long id);

    /**
     * 根据id集合查询员工信息列表
     *
     * @param ids id集合
     * @return 员工对象信息
     */
    List<EmployeeRespDTO> getEmployeeListByIds(Collection<Long> ids);

    /**
     * 根据用户id查询对应员工信息
     *
     * @param userId 用户ID
     * @return 员工对象信息
     */
    EmployeeRespDTO getEmployeeByUserId(Long userId);

    /**
     * 根据用户id查询对应员工信息
     *
     * @param userIds 用户ID
     * @return 员工对象信息
     */
    List<EmployeeRespDTO> getEmployeeListByUserIds(Collection<Long> userIds);

    /**
     * 创建员工
     *
     * @param reqDTO
     * @return
     */
    Long createEmployee(@Valid EmployeeCreateReqDTO reqDTO);

    /**
     * 查询员工
     *
     * @param reqDTO
     * @return
     */
    EmployeeRespDTO getEmployee(@Valid EmployeeQueryReqDTO reqDTO);

    /**
     * 查询员工分页
     *
     * @param pageReqVO
     * @return
     */
    PageResult<EmployeeRespDTO> getEmployeePage(@Valid EmployeePageReqDTO pageReqVO);
}
